library(gnlm)
library(growth)
library(repeated)
options(contrasts = c("contr.mean", "contr.poly"))

fq1 <- read.table("pkpd.dat",header=T)
attach(fq1)
tconc <-matrix(ifelse(conc>0,conc,0.04),ncol=12,byrow=T)
uconc <-matrix(ifelse(conc>0,conc,0.02),ncol=12,byrow=T)
cens <- matrix(ifelse(conc==0,-1,1),ncol=12,byrow=T)
t <- matrix(ifelse(time==0,0.01,time),ncol=12,byrow=T)
id <- 1:18
reps <- rmna(restovec(tconc,censor=cens,times=t,delta=0.01),ccov=tcctomat(id))
reps <- transform(reps,lconc=log(tconc))
repsuc <- rmna(restovec(uconc,times=t,delta=0.01),ccov=tcctomat(id))

if(interactive()){
postscript("pkpd.eps")
par(mar=c(3.5,3.5,3,1),mgp=c(2.5,1,0),font.main=1)
plot(reps,name="tconc",xlab="Hours",ylab="Concentration")
dev.off()
}

tconc1 <- matrix(ifelse(conc>0,conc,0.02),ncol=12,byrow=T)
reps2 <- rmna(restovec(tconc1,times=t,delta=0.01),ccov=tcctomat(id))

mu <- ~exp(ka-V)/(exp(ka)-exp(ke))*(exp(-exp(ke)*times)-exp(-exp(ka)*times))
lmu <- ~ka-V+log((exp(-exp(ke)*times)-exp(-exp(ka)*times))/(exp(ka)-exp(ke)))
shape <- ~d*(ka-V+log((exp(-exp(ke)*times)-exp(-exp(ka)*times))/
	(exp(ka)-exp(ke))))

f1mu <- ~exp(ka-(V+b1*(id==1)+b2*(id==2)+b3*(id==3)+b4*(id==4)+b5*(id==5)+
	b6*(id==6)+b7*(id==7)+b8*(id==8)+b9*(id==9)+b10*(id==10)+b11*(id==11)+
	b12*(id==12)+b13*(id==13)+b14*(id==14)+b15*(id==15)+
	b16*(id==16)+b17*(id==17)))/(exp(ka)-exp(ke))*
	(exp(-exp(ke)*times)-exp(-exp(ka)*times))
f1lmu <- ~ka-(V+b1*(id==1)+b2*(id==2)+b3*(id==3)+b4*(id==4)+b5*(id==5)+
	b6*(id==6)+b7*(id==7)+b8*(id==8)+b9*(id==9)+b10*(id==10)+b11*(id==11)+
	b12*(id==12)+b13*(id==13)+b14*(id==14)+b15*(id==15)+
	b16*(id==16)+b17*(id==17))+log((exp(-exp(ke)*times)-
	exp(-exp(ka)*times))/(exp(ka)-exp(ke)))
r1mu <- ~exp(ka-V-u)/(exp(ka)-exp(ke))*(exp(-exp(ke)*times)-
	exp(-exp(ka)*times))
r1lmu <- ~ka-V-u+log((exp(-exp(ke)*times)-exp(-exp(ka)*times))/
	(exp(ka)-exp(ke)))
f2mu <- ~{
	tmp <- b1*(id==1)+b2*(id==2)+b3*(id==3)+b4*(id==4)+b5*(id==5)+
		b6*(id==6)+b7*(id==7)+b8*(id==8)+b9*(id==9)+b10*(id==10)+
		b11*(id==11)+b12*(id==12)+b13*(id==13)+b14*(id==14)+
		b15*(id==15)+b16*(id==16)+b17*(id==17)
	exp(ka+tmp-V)/(exp(ka+tmp)-exp(ke))*
		(exp(-exp(ke)*times)-exp(-exp(ka+tmp)*times))}
f2lmu <- ~{
	tmp <- b1*(id==1)+b2*(id==2)+b3*(id==3)+b4*(id==4)+b5*(id==5)+
		b6*(id==6)+b7*(id==7)+b8*(id==8)+b9*(id==9)+b10*(id==10)+
		b11*(id==11)+b12*(id==12)+b13*(id==13)+b14*(id==14)+
		b15*(id==15)+b16*(id==16)+b17*(id==17)
	ka+tmp-V+log((exp(-exp(ke)*times)-exp(-exp(ka+tmp)*times))/
		(exp(ka+tmp)-exp(ke)))}
r2mu <- ~exp(ka+u-V)/(exp(ka+u)-exp(ke))*(exp(-exp(ke)*times)-
	exp(-exp(ka+u)*times))
r2lmu <- ~ka+u-V+log((exp(-exp(ke)*times)-exp(-exp(ka+u)*times))/
	(exp(ka+u)-exp(ke)))
f3mu <- ~{
	tmp <- b1*(id==1)+b2*(id==2)+b3*(id==3)+b4*(id==4)+b5*(id==5)+
		b6*(id==6)+b7*(id==7)+b8*(id==8)+b9*(id==9)+b10*(id==10)+
		b11*(id==11)+b12*(id==12)+b13*(id==13)+b14*(id==14)+
		b15*(id==15)+b16*(id==16)+b17*(id==17)
	exp(ka-V)/(exp(ka)-exp(ke+tmp))*
		(exp(-exp(ke+tmp)*times)-exp(-exp(ka)*times))}
f3lmu <- ~{
	tmp <- b1*(id==1)+b2*(id==2)+b3*(id==3)+b4*(id==4)+b5*(id==5)+
		b6*(id==6)+b7*(id==7)+b8*(id==8)+b9*(id==9)+b10*(id==10)+
		b11*(id==11)+b12*(id==12)+b13*(id==13)+b14*(id==14)+
		b15*(id==15)+b16*(id==16)+b17*(id==17)
	ka-V+log((exp(-exp(ke+tmp)*times)-exp(-exp(ka)*times))/
		(exp(ka)-exp(ke+tmp)))}
r3mu <- ~exp(ka-V)/(exp(ka)-exp(ke+u))*(exp(-exp(ke+u)*times)-
	exp(-exp(ka)*times))
r3mup <- ~exp(ka-V)/(exp(ka)-u*exp(ke))*(exp(-u*exp(ke)*times)-
	exp(-exp(ka)*times))
r3lmu <- ~ka-V+log((exp(-exp(ke+u)*times)-exp(-exp(ka)*times))/
	(exp(ka)-exp(ke+u)))

# normal mixing distribution
# normal distribution - constant variance
print(zn <- gnlr(lconc,"normal",pmu=c(1,0.65,-1),psh=0.1,mu=lmu,
	env=reps)) # 595.1579

print(znf1 <- gnlr(lconc,dist="normal",pmu=c(1,0.65,rep(0,17),-1),psh=-0.5,
	mu=f1lmu,env=reps),corr=F) # 559.7872
print(znh1 <- hnlmix(lconc,dist="normal",pmu=c(1,0.65,-1),psh=-0.5,mu=r1lmu,
	random="u",prandom=0,env=reps),corr=F) # 560.6249
print(zng1 <- gnlmix(lconc,dist="normal",pmu=znh1$coef[1:3],mu=r1lmu,
	psh=-0.54,pmix=-0.89,random="u",env=reps),corr=F) # 569.8197
print(zng1a <- gnlmm(lconc,dist="normal",pmu=c(1,0.65,-1),psh=-0.5,
	mu=lmu,psd=0.5,env=reps),corr=F) # 569.5711

print(znf2 <- gnlr(lconc,dist="normal",pmu=c(rep(0,17),1,0.65,-1),psh=-0.5,
	mu=f2lmu,env=reps),corr=F) # 556.8413
print(znh2 <- hnlmix(lconc,dist="normal",pmu=c(1,0.65,-1),psh=-0.5,mu=r2lmu,
	random="u",prandom=0,env=reps),corr=F) # 559.1276
print(zng2 <- gnlmix(lconc,dist="normal",pmu=c(-0.26,-0.35,-0.1),mu=r2lmu,
	psh=-0.34,pmix=-0.69,random="u",env=reps),corr=F) # 563.9547
#print(zng2 <- gnlmix(lconc,dist="normal",pmu=znh2$coef[1:3],mu=r2lmu,
#	psh=-1,pmix=-0.5,random="u",env=reps),corr=F) # 563.9547

print(znf3 <- gnlr(lconc,dist="normal",pmu=c(rep(0,17),1,0.65,-2),psh=-0.5,
	mu=f3lmu,env=reps),corr=F) # 467.9282
print(znh3 <- hnlmix(lconc,dist="normal",pmu=c(1,0.65,-1),psh=-0.5,mu=r3lmu,
	random="u",prandom=0,env=reps),corr=F) # 468.5609
print(zng3 <- gnlmix(lconc,dist="normal",pmu=c(0.18,-0.23,-0.15),mu=r3lmu,
	psh=-1.8,pmix=-0.8,random="u",env=reps),corr=F) # 492.3205
#print(zng3 <- gnlmix(lconc,dist="normal",pmu=znh3$coef[1:3],mu=r3lmu,
#	psh=-0.8,pmix=-0.5,random="u",env=reps),corr=F) # 492.3205

# normal distribution - variance function
print(znv <- gnlr(lconc,"normal",mu=lmu,shape=shape,pmu=c(1,0.65,-1),
	psh=c(-0.1,-0.8,-0.8,0.5),env=reps)) # 563.8932

print(znvf1 <- gnlr(lconc,dist="normal",mu=f1lmu,shape=shape,
	pmu=c(1,0.65,rep(0,17),-1),psh=c(-0.3,7,-4,-0.004),env=reps),corr=F)
	# 550.2222
print(znvh1 <- hnlmix(lconc,dist="normal",mu=r1lmu,shape=shape,
	pmu=c(1,0.65,-1),psh=c(-0.23,7,-4,-0.004),random="u",prandom=0,
	env=reps),corr=F) # 552.0374
print(znvg1 <- gnlmix(lconc,dist="normal",mu=r1lmu,shape=shape,
	pmu=c(0.7,0.22,-0.51),psh=c(-0.25,7,-5.57,-0.004),pmix=-1.83,
	random="u",env=reps),corr=F) # 555.5635
#	pmu=znh1$coef[1:3],psh=c(-0.23,7,-4,-0.004),pmix=-0.89,
#	random="u",env=reps),corr=F) # 555.5635
print(znvg1a <- gnlmm(lconc,dist="normal",mu=lmu,shape=shape,
	pmu=c(1,0.65,-1),psh=c(-0.23,7,-4,-0.004),psd=0.5,env=reps),corr=F)
	# 555.5754

print(znvf2 <- gnlr(lconc,dist="normal",mu=f2lmu,shape=shape,
	pmu=c(rep(0,17),1,0.65,-1),psh=c(-0.23,7,-4,-0.004),env=reps),
	corr=F) # 545.2225
print(znvh2 <- hnlmix(lconc,dist="normal",mu=r2lmu,shape=shape,
	pmu=c(1,0.65,-1),psh=c(-0.23,7,-4,-0.004),random="u",prandom=0,
	env=reps),corr=F) # 547.4018
print(znvg2 <- gnlmix(lconc,dist="normal",mu=r2lmu,shape=shape,
	pmu=c(0.16,-0.36,0.05),psh=c(-0.26,8.2,-5,-0.004),pmix=-1.32,
	random="u",env=reps),corr=F) # 549.0215
#	pmu=znh2$coef[1:3],psh=c(-0.23,7,-4,-0.004),pmix=-0.5,random="u",
#	env=reps),corr=F) # 549.0215

print(znvf3 <- gnlr(lconc,dist="normal",mu=f3lmu,shape=shape,
	pmu=c(rep(0,17),0.2,-0.2,-1.9),psh=c(-0.23,7,-4,-0.004),env=reps),
	corr=F) # 471.3653
print(znvh3 <- hnlmix(lconc,dist="normal",mu=r3lmu,shape=shape,
	pmu=c(1,0.65,-1),psh=c(-0.23,7,-4,-0.004),random="u",prandom=0,
	env=reps),corr=F) # 472.8781
print(znvg3 <- gnlmix(lconc,dist="normal",mu=r3lmu,shape=shape,
	pmu=znh3$coef[1:3],psh=c(-0.23,7,-4,-0.004),pmix=-0.5,random="u",
	env=reps),corr=F) # 

# gamma distribution - constant dispersion
print(zg <- gnlr(tconc,"gamma",pmu=c(1,0.65,-1),psh=0.1,mu=mu,
	env=reps)) # 610.9427

print(zgf1 <- gnlr(tconc,dist="gamma",pmu=c(1,0.65,rep(0,17),-1),psh=-0.5,
	mu=f1mu,env=reps),corr=F) # 545.2913
print(zgh1 <- hnlmix(tconc,dist="gamma",pmu=c(1,0.65,-1),psh=-0.5,mu=r1mu,
	random="u",prandom=0,env=reps),corr=F) # 546.6077
print(zgg1 <- gnlmix(tconc,dist="gamma",pmu=znh1$coef[1:3],mu=r1mu,
	psh=-0.54,pmix=-0.89,random="u",env=reps),corr=F) # 
print(zgg1a <- gnlmm(tconc,dist="gamma",pmu=c(1.22,0.5,-1),psh=0.62,
	mu=mu,psd=1,env=reps,points=3),corr=F) # 554.2053
print(zgg1a <- gnlmm(tconc,dist="gamma",pmu=c(1.5,-0.47,-1.07),psh=0.73,
	mu=mu,psd=1.2,env=reps,points=2),corr=F) # 553.4688

print(znf2 <- gnlr(tconc,dist="gamma",pmu=c(rep(2,17),0.5,0,-1.8),psh=0.3,
	mu=f2mu,env=reps),corr=F) # 573.8481
print(zgh2 <- hnlmix(tconc,dist="gamma",pmu=c(0.5,0,-1.8),psh=0.3,mu=r2mu,
	random="u",prandom=rep(-1,18),env=reps),corr=F) # 627.7283
print(zgg2 <- gnlmix(tconc,dist="gamma",pmu=zgh2$coef[1:3],mu=r2mu,
	psh=-1,pmix=-0.5,random="u",env=reps),corr=F) # 

print(zgf3 <- gnlr(tconc,dist="gamma",pmu=c(rep(0,17),1,0.65,-2),psh=-0.5,
	mu=f3mu,env=reps),corr=F) # 451.1074
print(zgh3 <- hnlmix(tconc,dist="gamma",pmu=c(1,0.65,-1),psh=-0.5,mu=r3mu,
	random="u",prandom=0,env=reps),corr=F) # 451.4025
print(zgg3 <- gnlmix(tconc,dist="gamma",pmu=zgh3$coef[1:3],mu=r3mu,
	psh=-0.8,pmix=-0.5,random="u",env=reps),corr=F) # 

# gamma distribution - dispersion function
print(zgv <- gnlr(tconc,"gamma",mu=mu,shape=shape,pmu=c(1.3,0.3,-1),
	psh=c(1,2,0,-4),env=reps,iter=200)) # 548.8748

print(zgvf1 <- gnlr(tconc,dist="gamma",mu=f1mu,shape=shape,
	pmu=c(2,0.65,rep(0,17),-1),psh=c(1,2,0,-4),env=reps),corr=F) # 490.8422
print(zgvh1 <- hnlmix(tconc,dist="gamma",mu=r1mu,shape=shape,
	pmu=c(1,0.65,-1),psh=c(1,2,0,-4),random="u",prandom=0,
	env=reps),corr=F) # 491.5506
print(zgvg1 <- gnlmix(tconc,dist="gamma",mu=r1mu,shape=shape,
	pmu=zgh1$coef[1:3],psh=c(1,2,0,-4),pmix=-0.89,
	random="u",env=reps),corr=F) # 
print(zgvg1a <- gnlmm(tconc,dist="gamma",mu=mu,shape=shape,
	pmu=c(1.5,0.37,-0.7),psh=c(6,1,-0.4,-2.5),psd=0.8,env=reps,points=3),
	corr=F) # 525.7187, 5 points: 527.7163

print(zgvf2 <- gnlr(tconc,dist="gamma",mu=f2mu,shape=shape,
	pmu=c(rep(0.5,17),1.3,0.3,-1),psh=c(6,2,0,-4),env=reps,iter=300),
	corr=F) # 543.4168
print(zgvh2 <- hnlmix(tconc,dist="gamma",mu=r2mu,shape=shape,
	pmu=c(1,0.65,-1),psh=c(6,1,-0.4,-2.5),random="u",prandom=0,
	env=reps,iter=200),corr=F) # 565.5542
print(zgvg2 <- gnlmix(tconc,dist="gamma",mu=r2mu,shape=shape,
	pmu=zgh2$coef[1:3],psh=c(6,1,-0.4,-2.5),pmix=-0.5,random="u",
	env=reps),corr=F) # 

print(zgvf3 <- gnlr(tconc,dist="gamma",mu=f3mu,shape=shape,
	pmu=c(rep(1,17),0.2,-0.2,-1.9),psh=c(6,1,-0.4,-2.5),env=reps,iter=200),
	corr=F) # 426.5976
c(zgvf3$coef[1:17],0)-sum(zgvf3$coef[1:17])/18
print(zgvh3 <- hnlmix(tconc,dist="gamma",mu=r3mu,shape=shape,
	pmu=c(0.003,-0.34,-0.05),psh=c(6,1.3,-0.23,-2.5),random="u",
	prandom=0,env=reps,iter=200),corr=F) # 426.7284
print(zgvg3 <- gnlmix(tconc,dist="gamma",mu=r3mu,shape=shape,
	pmu=c(-0.037,-0.38,0.14),psh=c(8.2,1.45,-0.18,-8.14),pmix=-0.8,
	random="u",env=reps,steps=6),corr=F) # 460.9224

print(zgvuc <- gnlr(uconc,dist="gamma",mu=mu,shape=shape,
	pmu=c(1.86,1.14,-2.05),psh=c(-6.31,1.55,0.02,-15),
	env=repsuc,iter=200),corr=F) # 615.9849
print(zgvf3uc <- gnlr(uconc,dist="gamma",mu=f3mu,shape=shape,
	pmu=c(rep(0.05,17),1.86,1.14,-2.05),psh=c(-6.31,1.55,0.02,-15),
	env=repsuc,iter=200),corr=F) # 632.0636
print(zgvh3uc <- hnlmix(uconc,dist="gamma",mu=r3mu,shape=shape,
	pmu=c(1.86,1.14,-2.05),psh=c(-6.31,1.55,0.02,-15),random="u",
	prandom=0.01,env=repsuc,iter=200),corr=F) # 631.5322

# gamma mixing distribution
# gamma distribution - dispersion function
print(zgvhg3 <- hnlmix(tconc,dist="gamma",mix="gamma",mu=r3mup,shape=shape,
	pmu=c(-0.01,-0.35,-0.14),psh=c(9,1.2,-0.26,-10),random="u",prandom=1,
	env=reps,iter=300),corr=F) # 425.7457
print(zgvgg3 <- gnlmix(tconc,dist="gamma",mix="gamma",mu=r3mup,shape=shape,
	pmu=c(-0.037,-0.38,0.14)),psh=c(17.6,1.44,-0.17,-18.14)),pmix=1.08,
	random="u",env=reps),corr=F) # 458.7267
#	pmu=zgv$coef[1:3],psh=c(6,1,-0.4,-2.5),pmix=0,random="u",
#	env=reps),corr=F) # 

if(interactive()){
par(mfrow=c(2,2))
plot(iprofile(zgvh3),nind=1)
plot(iprofile(zgvhg3),nind=1,lty=2,add=T)
lines(t[18,],finterp(f3mu,.env=reps)(zgvf3$coef)[1:12],lty=3)
plot(iprofile(zgvh3),nind=2)
plot(iprofile(zgvhg3),nind=2,lty=2,add=T)
lines(t[18,],finterp(f3mu,.env=reps)(zgvf3$coef)[13:24],lty=3)
plot(iprofile(zgvh3),nind=12)
plot(iprofile(zgvhg3),nind=12,lty=2,add=T)
lines(t[18,],finterp(f3mu,.env=reps)(zgvf3$coef)[133:144],lty=3)
plot(iprofile(zgvh3),nind=18)
plot(iprofile(zgvhg3),nind=18,lty=2,add=T)
lines(t[18,],finterp(f3mu,.env=reps)(zgvf3$coef)[205:216],lty=3)
}

# Weibull mixing distribution
# gamma distribution - dispersion function
print(zgvhw3 <- hnlmix(tconc,dist="gamma",mix="Weibull",mu=r3mup,shape=shape,
	pmu=c(1,0.65,-1),psh=c(6,1,-0.4,-2.5),random="u",prandom=1,
	env=reps,iter=300),corr=F) # 429.1127
print(zgvgw3 <- gnlmix(tconc,dist="gamma",mix="Weibull",mu=r3mup,shape=shape,
	pmu=znh3$coef[1:3],psh=c(6,1,-0.4,-2.5),pmix=-0.5,random="u",
	env=reps),corr=F) # 
