library(gnlm)
library(repeated)

data <- read.table("seeds.dat",skip=4,head=T)
data <- cbind(data,id=c(rep(1,5),rep(2,6),rep(3,5),rep(4,5)),plate=1:21)
data$variety <- 2-codes(data$variety)
data$extract <- codes(data$extract)-1
reps <- dftorep(data,response="r",total="n",id="id",
	ccov=c("variety","extract"),tvcov="plate")
reps2 <- dftorep(data,response="r",total="n",
	ccov=c("variety","extract","plate"))

print(z1 <- glm(cbind(r,n-r)~variety*extract,family=binomial,data=data))
z1$aic/2 # 58.93702
gnlr(reps,dist="binomial",mu=~1/(1+exp(-linear)),linear=~variety*extract,
	pmu=c(1,0,0,0)) # 58.93702
gnlr(reps,dist="beta binomial",mu=~1/(1+exp(-linear)),linear=~variety*extract,
	pmu=c(1,0,0,0),psh=0) # 58.76676

print(z2 <- glmm(cbind(r,n-r)~variety*extract,nest=id,family=binomial,
	data=data))
z2$aic/2 # 59.77084
print(z3 <- glmm(cbind(r,n-r)~variety*extract,nest=plate,family=binomial,
	data=data))
z3$aic/2 # 58.59253

hnlmix(reps2,dist="binomial",mixture="beta",
	mu=~a/(1+exp(-(b0+b1*variety+b2*extract+b3*variety*extract))),
	pmu=c(-0.54,0.1,1.3,-0.8),prandom=0.5,random="a") # 62.93702
hnlmix(reps,dist="binomial",mixture="beta",
	mu=~1/(1+exp(-(b0+b1*variety+b2*extract+b3*variety*extract))),
	pmu=c(0.1,1.8,-0.71),prandom=-0.5,random="b0")
	# 68.35169
hnlmix(reps,dist="binomial",mixture="beta",
	mu=~1/(1+exp(-(b0+b1*variety+b2*extract+b3*variety*extract))),
	pmu=c(0.1,1.8,-0.71),pmix=4.381,prandom=-0.5,random="b0")
	# 62.63771

hnlmix(reps,dist="beta binomial",mixture="beta",psh=0,
	mu=~1/(1+exp(-(b0+b1*variety+b2*extract+b3*variety*extract))),
	pmu=c(0.1,1.8,-0.71),prandom=-0.5,random="b0") # 62.76677
hnlmix(reps,dist="beta binomial",mixture="beta",psh=0,
	mu=~1/(1+exp(-(b0+b1*variety+b2*extract+b3*variety*extract))),
	pmu=c(0.1,1.8,-0.71),pmix=3.096,prandom=-0.5,random="b0")
	# 68.14337

print(u0 <- hnlmix(reps2,dist="binomial",mixture="beta",pmu=3,
	mu=~b0/(1+exp(-a)),prandom=0.4,random="b0"),corr=F) # 79.29698
print(u1 <- hnlmix(reps2,dist="binomial",mixture="beta",
	mu=~1/(1+exp(-(b0))),
	pmix=3.096,prandom=-0.5,random="b0",disp="fixed"),corr=F) # 79.29698
print(u2 <- hnlmix(reps2,dist="binomial",mixture="beta",
	mu=~1/(1+exp(-(b0+b1*variety+b2*extract+b3*variety*extract))),
	pmu=c(0.1,1.8,-0.71),pmix=3.096,prandom=-0.5,random="b0",disp="fixed",
	iter=200),corr=F) # 79.29698
u2$aic-3 # 71.44018
print(z4 <- glm(cbind(r,n-r)~as.factor(plate)-1,family=binomial,data=data))
z4$aic/2 # 59.29814
cbind(u1$coef,z4$coef)
cbind(data$r,data$n/(1+exp(-(u2$coef[4:24]+u2$coef[1]*data$variety
	+u2$coef[2]*data$extract+u2$coef[3]*data$variety*data$extract))))
cbind(u2$coef[4:24]-mean(u2$coef[4:24]),z4$coef)
