library(repeated)
library(growth)
library(gnlm)
options(contrasts=c("contr.mean","contr.poly"))

# 4 units, 5 responses/unit
dose <- c(9,12,4,9,11,10,2,11,12,9,9,9,4,9,11,9,14,7,9,8)
#u <- round(rnorm(20,2+0.3*dose,4),1)
u <- c(7.7,1.8,5.7,0.2,4.8,-2.3,9.6,4.7,11.2,3.8,5.3,-2.7,8.4,5.4,4.5,0.4,
	5.9,-0.7,4.1,7.7)
#v <- round(rnorm(4,0,4),1)
v <- c(-2.2,1.9,10.1,0.6)
y <- u+rep(v,rep(5,4))
resp <- restovec(matrix(y, nrow=4, byrow=T), name="y", delta=0.1)
reps <- rmna(resp, tvcov=tvctomat(matrix(dose, nrow=4, byrow=T), name="dose"))

gnlr(reps, mu=~dose, pmu=c(2,0.3), psh=1) # 112.7215
gnlr(reps, mu=~dose+individuals, pmu=c(2,0.3,0,0,0), psh=1) # 106.0615
elliptic(reps, model=~dose+individuals, preg=c(9,0.6,0,0,0)) # 106.0615
elliptic(reps, model=~dose, preg=c(0,0.6), pre=4) # 110.0585
hnlmix(reps, mu=~a+rand+b*dose, random="rand", pmu=c(10,0.25),
	pshape=3.44, prandom=c(3,-4,-3,4)) # 106.2479

# 4 units, 10 responses/unit
dose2 <- rep(dose,2)
#u2 <- round(rnorm(40,2+0.3*dose2,4),1)
u2 <- c(5.8,-2.3,0.7,2.7,5.8,2.5,3.6,8.9,6.0,6.0,7.1,7.6,-1.6,8.4,4.0,9.9,
	7.4,2.8,7.1,4.7,2.1,2.3,7.7,4.7,13.2,-1.0,-4.1,3.7,8.9,7.8,7.5,2.5,
	4.9,2.9,11.1,0.6,12.8,3.4,3.6,4.7)
#v2 <- round(rnorm(4,0,4),1)
v2 <- c(3.7,-1.2,-3.4,-2.4)
y2 <- u2+rep(v2,rep(10,4))
resp2 <- restovec(matrix(y2, nrow=4, byrow=T), name="y", delta=0.1)
reps2 <- rmna(resp2,tvcov=tvctomat(matrix(dose2,nrow=4,byrow=T),name="dose"))

gnlr(reps2, mu=~dose, pmu=c(2,0.3), psh=1) # 208.7441
gnlr(reps2, mu=~dose+individuals, pmu=c(2,0.3,0,0,0), psh=1) # 203.6119
elliptic(reps2, model=~dose+individuals, preg=c(9,0.6,0,0,0)) # 203.6119
elliptic(reps2, model=~dose, preg=c(0,0.6), pre=4) # 206.7341
hnlmix(reps2, mu=~a+rand+b*dose, random="rand", pmu=c(10,0.25),
	pshape=3.44, prandom=c(3,-4,-3,4)) # 203.7175

# 8 units, 5 responses/unit
dose3 <- dose2
u3 <- u2
#v3 <- round(rnorm(8,0,4),1)
v3 <- c(0.3,-2.4,-8.1,-0.7,-3.0,3.6,-4.8,-9.0)
y3 <- u3+rep(v3,rep(5,8))
resp3 <- restovec(matrix(y3, nrow=8, byrow=T), name="y", delta=0.1)
reps3 <- rmna(resp3,tvcov=tvctomat(matrix(dose2,nrow=8,byrow=T),name="dose"))

gnlr(reps3, mu=~dose, pmu=c(2,0.3), psh=1) # 214.1655
gnlr(reps3, mu=~dose+individuals, pmu=c(2,0.3,rep(0,7)), psh=1) # 205.4073
elliptic(reps3, model=~dose+individuals, preg=c(9,0.6,rep(0,7))) # 205.4073
elliptic(reps3, model=~dose, preg=c(0,0.6), pre=4) # 210.141
hnlmix(reps3, mu=~a+rand+b*dose, random="rand", pmu=c(10,0.25),
	pshape=3.44, prandom=0) # 205.8210
